local ldb_tip
local frame   = CreateFrame("Button", "Haggler")
local ldb_obj = LibStub("LibDataBroker-1.1"):NewDataObject("Haggler", {
	label    = "Haggler",
	type     = "launcher"
})

HagglerDB = HagglerDB or { junkie = false, auto = false }

frame:RegisterEvent"MERCHANT_SHOW"
frame:RegisterEvent"MERCHANT_CLOSED"

frame:SetScript("OnEvent", function(_, event, ...) frame[event](...) end)

local function inform(text)
	DEFAULT_CHAT_FRAME:AddMessage("|cffffd700Haggler:|r |cffeeeeee"..text.."|r")
end

local function cash_to_string(cash)
	if not cash then return "?" end

	local gold   = floor(cash / (100 * 100))
	local silver = math.fmod(floor(cash / 100), 100)
	local copper = math.fmod(floor(cash), 100)
	gold         = gold   > 0 and "|cffeeeeee"..gold  .."|r|cffffd700g|r" or ""
	silver       = silver > 0 and "|cffeeeeee"..silver.."|r|cffc7c7cfs|r" or ""
	copper       = copper > 0 and "|cffeeeeee"..copper.."|r|cffeda55fc|r" or ""
	copper       = (silver ~= "" and copper ~= "") and " "..copper or copper
	silver       = (gold   ~= "" and silver ~= "") and " "..silver or silver

	return gold..silver..copper
end

local function sell()
	if not MerchantFrame:IsVisible() then return end

	local sold = 0

	inform("Passing your junk to the vendor for moneys...")

	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)

			if link then
				local _, count      = GetContainerItemInfo(bag, slot)
				local _, _, quality = GetItemInfo(link)
				local id            = tonumber(link:match("item:(%d+)"))

				if (HagglerDB.junkie and JunkieDB:find("-"..id)) or (quality and quality == 0) then
					local price = (GetSellValue(link) or 0)

					if price > 0 then
						sold = sold + price
		
						ShowMerchantSellCursor(1)
						UseContainerItem(bag, slot)
		
						inform("Selling "..link..((count or 0) > 1 and "x"..count or "").." to vendor for "..cash_to_string(price)..".")
					end
				end
			end
		end
	end

	sold = sold > 0 and "seems pleased enough with your junk to give you "..cash_to_string(sold) or "seems dismayed at your lack of junk"

	inform("The vendor "..sold..".")
end

local function shop_show()
	ldb_obj.icon = "Interface\\Icons\\INV_Misc_Coin_02.png"

	if HagglerDB.auto then sell() end
end

local function shop_hide()
	ldb_obj.icon = nil
end

function ldb_obj.OnTooltipShow(tip)
	if not ldb_tip then ldb_tip = tip end

	tip:ClearLines()

	tip:AddLine("Haggler: A DataBroker Plugin")
	tip:AddLine(" ")

	tip:AddLine("Auto-Sell:|r |cffeeeeee"..(HagglerDB.auto and "|cff88ff88Enabled" or "|cffff8888Disabled").."|r")

	if JunkieDB then
		tip:AddLine("Selling:|r |cffeeeeeeTrash"..(HagglerDB.junkie and " and Junkie exceptions" or "")..".|r")
		tip:AddLine(" ")
	end

	tip:AddLine("|cff69b950Click:|r |cffeeeeeeSell your trash items to the vendor.|r")

	if JunkieDB then
		tip:AddLine("|cff69b950Alt + Click:|r |cffeeeeeeToggles selling Junkie exceptions.|r")
	end

	tip:AddLine("|cff69b950Shift + Click:|r |cffeeeeeeToggles vendor auto-selling.|r")

	tip:Show()
end

function ldb_obj.OnClick()
	if IsAltKeyDown() then
		HagglerDB.junkie = not HagglerDB.junkie

		ldb_obj.OnTooltipShow(ldb_tip)

	elseif IsShiftKeyDown() then
		HagglerDB.auto = not HagglerDB.auto

		ldb_obj.OnTooltipShow(ldb_tip)

	else sell() end
end

frame.MERCHANT_SHOW   = shop_show
frame.MERCHANT_CLOSED = shop_hide